def parse(): ...


def serialize(): ...


def open(): ...


def read(): ...


def save(): ...


def load(): ...


def stream(): ...


def download(): ...


# def append(): ...


# def prepend(): ...


# def walk(): ...


# import orjson
# from pyeio.types import FilePath


# def stream(): ...


# # import json
# # from pathlib import Path
# # from .core import io


# # def load(path: str | Path) -> list | dict:
# #     return json.loads(io.load_text(path))


# # def save(): ...


# # todo
# # def stream(
# #     path: str | Path,
# #     validator,
# #     handler,
# # ): ...


# # # same as stream except operation is done on each element
# # def stream_ingest(
# #     path,
# #     validator,
# #     handler,
# #     errors,  # raise or log
# # ): ...
