from importlib.metadata import version

__name__ = __package__ = __module__ = "pyeio"
__version__ = version(__name__)
__repository__ = "https://github.com/harttraveller/pyeio"
__documentation__ = "https://harttraveller.github.io/pyeio/"
__issues__ = "https://github.com/harttraveller/pyeio/issues/"
